#include <formatio.h>
#include <rs232.h>
#include <cvirte.h>		
#include <userint.h>
#include "bios.h"

static int panelHandle;
static int fileHandle;

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "bios.uir", PANEL)) < 0)
		return -1;
	fileHandle=OpenFile ("AV18S403.ROM", VAL_READ_ONLY, VAL_OPEN_AS_IS, VAL_BINARY);
	OpenComConfig (1, "", 19200, 0, 8, 1, 2, 2);

	DisplayPanel (panelHandle);
	RunUserInterface ();
	DiscardPanel (panelHandle);
	CloseCom (1);
	CloseFile (fileHandle);

	return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
			QuitUserInterface (0);  
			break;
		}
	return 0;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
		}
	return 0;
}

void WriteByte(unsigned char temp)
	{
	unsigned char reply;
	SetCtrlVal (panelHandle, PANEL_LED, 0);
	do
		{
		ComWrtByte (1, temp);
		ComWrtByte (1, temp);
		reply = ComRdByte (1);
		if(reply=='T') SetCtrlVal (panelHandle, PANEL_LED, 1);
		}
	while(reply!='O');	//data accepted
	SetCtrlVal (panelHandle, PANEL_LED, 0);
	
	}

unsigned char receive_byte(void)
	{
	unsigned char temp,prevRSbyte,stan=0;
	unsigned char accepted=0;
	while(accepted==0)	//dopoki nie zaakceptowano danej
		{
		if(stan==0) 
			{
					   
			prevRSbyte=ComRdByte (1);
			stan=1;
			}
		if(stan==1)
			{
			//while(usart_unread_data()==0){};	//wait for data
			temp=ComRdByte (1);
			stan=0;
			}
			
		if(temp==prevRSbyte)
			{
			accepted=1;
			//sendACK();
			ComWrtByte (1, 'O');
			}
		else
			{
			accepted=0;	
			//sendNACK();
			ComWrtByte (1, 'T');
			
			}
		}
	return temp;	//zwroc zaakcetowany bajt
	}

int CVICALLBACK ProgramCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	unsigned char data;
	//unsigned char buffer[10];
	
	int adres=0;
	
	switch (event)
		{
		case EVENT_COMMIT:
			FlushInQ (1);
			FlushOutQ (1);
			
			//odblokowanie i kasowanie - wyslanie ciagu ABC
			do
				{
				ComWrtByte (1, 'A');
				ComWrtByte (1, 'B');
				ComWrtByte (1, 'C');
				}
			while(ComRdByte (1)!='O');
			
			do {} while(ComRdByte (1)!='O');	//ACK po kasowaniu
			
			FlushInQ (1);
			FlushOutQ (1);
			
			//programowanie
			for(adres=0;adres<262144;adres++)
				{
				//pobranie danej
				SetFilePtr (fileHandle, adres, 0);
				ReadFile (fileHandle, &data, 1);
	
				SetCtrlVal (panelHandle, PANEL_NUMERIC, adres);
				SetCtrlVal (panelHandle, PANEL_NUMERIC_2, data);
				//wysylanie danej
				WriteByte(data);
				
				
				
				}
			break;
		}
	return 0;
}

int CVICALLBACK ReadCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	static int fH;	//file handle
	int adres;
	unsigned char temp;
	
	
	fH=OpenFile ("read.rom", VAL_READ_WRITE, VAL_APPEND, VAL_BINARY);
	
	switch (event)
		{
		case EVENT_COMMIT:
			FlushInQ (1);
			FlushOutQ (1);
			//odczyt wyslanie R
			do
				{
				ComWrtByte (1, 'R');
				}
			while(ComRdByte (1)!='O');
			
			//do {} while(ComRdByte (1)!='O');	//ACK po kasowaniu
			
			//FlushInQ (1);
			//FlushOutQ (1);
			
			//odczyt
			for(adres=0;adres<262144;adres++)
				{
				SetCtrlVal (panelHandle, PANEL_NUMERIC, adres); 
				temp=receive_byte();
				SetCtrlVal (panelHandle, PANEL_NUMERIC_2, temp);
				//dopisz bajt do pliku
				WriteFile (fH, &temp, 1);
				}

			break;
		}
	CloseFile (fH);
	return 0;
}

int CVICALLBACK EraseCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			FlushInQ (1);
			FlushOutQ (1);
			//odczyt wyslanie R
			do
				{
				ComWrtByte (1, 'E');
				}
			while(ComRdByte (1)!='O');
			
			break;
		}
	return 0;
}
